package FCC::Class::HTTP::Cookie;
$VERSION = 1.00;
use strict;
use warnings;
use CGI::Cookie;

sub new {
	my($caller, %args) = @_;
	my $class = ref($caller) || $caller;
	my $self = {};
	$self->{name} = $args{"-name"};
	$self->{value} = $args{"-value"};
	$self->{expires} = $args{"-expires"};
	$self->{domain} = $args{"-domain"};
	$self->{path} = $args{"-path"};
	$self->{secure} = $args{"-secure"};
	$self->{httponly} = $args{"-httponly"};
	bless $self, $class;
	return $self;
}

sub as_string {
	my($self) = @_;
	my $c = new CGI::Cookie(
		-name     => $self->{name},
		-value    => $self->{value},
		-expires  => $self->{expires},
		-domain   => $self->{domain},
		-path     => $self->{path},
		-secure   => $self->{secure},
		-httponly => $self->{httponly}
	);
	my $s = $c->as_string();
	my @pairs = split(/\;\s*/, $s);
	my @ary;
	for my $p (@pairs) {
		unless($p) { next; }
		my($k, $v) = split(/\=/, $p);
		if($self->{lc($k)} || $k eq $self->{name}) {
			push(@ary, $p);
		}
	}
	my $cs = join("; ", @ary);
	if($self->{httponly} && $cs !~ /httponly/i) {
		if($cs !~ /\;$/) {
			$cs .= ";";
		}
		$cs .= " httpOnly";
	}
	return $cs;
}


1;
